import flask
from openid_teams import teams
from werkzeug.wrappers import Response
from urllib.parse import urlparse, urljoin

from pyfaf.storage.user import User
from webfaf.webfaf_main import db, oid, app
from webfaf.utils import fed_raw_name
from onelogin.saml2.auth import OneLogin_Saml2_Auth

login = flask.Blueprint('login', __name__)


def init_saml_auth(req):
    saml_config_dir = app.config['SAML_CONFIG_DIR']
    return OneLogin_Saml2_Auth(req, custom_base_path=saml_config_dir)


def prepare_flask_request(req):
    parsed_url = urlparse(req.url)
    return {
        'https': 'on' if req.scheme == 'https' else 'off',
        'http_host': req.host,
        'server_port': parsed_url.port,
        'script_name': req.script_root + req.path,
        'get_data': req.args.copy(),
        'post_data': req.form.copy(),
        'query_string': req.query_string
    }


@login.route('/login/', methods=('GET',))
def do_login():
    """SSO authentication entry-point."""
    req = prepare_flask_request(flask.request)
    comeback_url = urljoin(flask.request.host_url, '/faf/summary')
    return_to = flask.request.args.get('return_to')
    if return_to:
        comeback_url += '?return_to={0}'.format(return_to)
    auth = init_saml_auth(req)
    return flask.redirect(auth.login(comeback_url))


@login.route('/acs/', methods=('POST',))
def acs():
    req = prepare_flask_request(flask.request)
    auth = init_saml_auth(req)
    auth.process_response()
    errors = auth.get_errors()
    if errors:
        return flask.helpers.make_response('Error: {0}'.format(', '.join(errors)), 500)
    elif not auth.is_authenticated():
        return flask.helpers.make_response('Error: authentication failed', 401)
    user_attrs = auth.get_attributes()

    privileged = False
    is_admin = False
    if any(group in user_attrs['groups'] for group in ('alma_retrace_admin', 'admins')):
        privileged = True
        is_admin = True

    email = user_attrs['email'][0]
    username = email.split('@')[0]
    user = db.session.query(User).filter(User.username == username).first()
    if not user:  # create
        user = User(username=username, mail=email, privileged=privileged, admin=is_admin)
    else:
        user.mail = email
        user.privileged = privileged
        user.admin = is_admin

    db.session.add(user)
    db.session.commit()
    flask.flash(u"Welcome, {0}".format(user.username))
    flask.g.user = user
    flask.session['username'] = username

    return flask.redirect(flask.request.form['RelayState'])


@login.route('/metadata/', methods=('GET',))
def metadata():
    """
    Returns the Build System IDP provider metadata.

    Returns
    -------
    flask.wrappers.Response
        IDP provider metadata response.

    See Also
    --------
    https://en.wikipedia.org/wiki/SAML_Metadata#Identity_Provider_Metadata
    """
    req = prepare_flask_request(flask.request)
    auth = init_saml_auth(req)
    settings = auth.get_settings()
    metadata = settings.get_sp_metadata()
    errors = settings.validate_metadata(metadata)
    if errors:
        rsp = flask.make_response(', '.join(errors), 500)
    else:
        rsp = flask.make_response(metadata, 200)
        rsp.headers['Content-Type'] = 'text/xml'
    return rsp


@login.route("/logout/")
def do_logout() -> Response:
    flask.session.pop("username", None)
    flask.flash(u"You were signed out")
    return flask.redirect(flask.url_for("summary.index"))
